@@ -10,8 +10,5 @@
 import sys
 
 if __name__ == '__main__':
-    print 'sphinx.web currently doesn\'t work -- it will undergo a ' \
-          'serious rewrite soon.'
-    sys.exit()
     from sphinx.web import main
     sys.exit(main(sys.argv))
