@@ -45,6 +45,9 @@ class Builder(object):
     Builds target formats from the reST sources.
     """
 
+    # builder's name, for the -b command line options
+    name = ''
+
     def __init__(self, app, env=None, freshenv=False):
         self.srcdir = app.srcdir
         self.outdir = app.outdir
@@ -146,7 +149,7 @@ class Builder(object):
 
     def build_all(self):
         """Build all source files."""
-        self.build(None, summary='all source files')
+        self.build(None, summary='all source files', method='all')
 
     def build_specific(self, filenames):
         """Only rebuild as much as needed for changes in the source_filenames."""
@@ -159,14 +162,17 @@ class Builder(object):
             filename = path.abspath(filename)[dirlen:]
             if filename.endswith(suffix):
                 filename = filename[:-len(suffix)]
+            filename = filename.replace(os.path.sep, SEP)
             to_write.append(filename)
-        self.build(to_write,
-                   summary='%d source files given on command line' % len(to_write))
+        self.build(to_write, method='specific',
+                   summary='%d source files given on command '
+                   'line' % len(to_write))
 
     def build_update(self):
         """Only rebuild files changed or added since last build."""
         to_build = self.get_outdated_docs()
-        if not to_build:
+        if not to_build and self.env.all_docs:
+            # if there is nothing in all_docs, it's a fresh env
             self.info(bold('no target files are out of date, exiting.'))
             return
         if isinstance(to_build, str):
@@ -177,7 +183,7 @@ class Builder(object):
                        summary='targets for %d source files that are '
                        'out of date' % len(to_build))
 
-    def build(self, docnames, summary=None):
+    def build(self, docnames, summary=None, method='update'):
         if summary:
             self.info(bold('building [%s]: ' % self.name), nonl=1)
             self.info(summary)
@@ -210,7 +216,7 @@ class Builder(object):
 
         # another indirection to support methods which don't build files
         # individually
-        self.write(docnames, updated_docnames)
+        self.write(docnames, updated_docnames, method)
 
         # finish (write static files etc.)
         self.info(bold('finishing... '))
@@ -220,10 +226,15 @@ class Builder(object):
         else:
             self.info(bold('build succeeded.'))
 
-    def write(self, build_docnames, updated_docnames):
-        if build_docnames is None: # build_all
+    def write(self, build_docnames, updated_docnames, method='update'):
+        if build_docnames is None:
+            # build_all
             build_docnames = self.env.all_docs
-        docnames = set(build_docnames) | set(updated_docnames)
+        if method == 'update':
+            # build updated ones as well
+            docnames = set(build_docnames) | set(updated_docnames)
+        else:
+            docnames = set(build_docnames)
 
         # add all toctree-containing files that may have changed
         for docname in list(docnames):
@@ -231,8 +242,6 @@ class Builder(object):
                 docnames.add(tocdocname)
         docnames.add(self.config.master_doc)
 
-        self.info(bold('creating index...'))
-        self.env.create_index(self)
         self.prepare_writing(docnames)
 
         # write target files
@@ -293,6 +302,10 @@ class StandaloneHTMLBuilder(Builder):
 
     def prepare_writing(self, docnames):
         from sphinx.search import IndexBuilder
+
+        self.info(bold('creating index...'))
+        self.env.create_index(self)
+
         self.indexer = IndexBuilder()
         self.load_indexer(docnames)
         self.docwriter = HTMLWriter(self)
