@@ -57,7 +57,7 @@ default_settings = {
 
 # This is increased every time a new environment attribute is added
 # to properly invalidate pickle files.
-ENV_VERSION = 17
+ENV_VERSION = 18
 
 
 def walk_depth(node, depth, maxdepth):
@@ -129,6 +129,19 @@ class MoveModuleTargets(Transform):
                 node.parent.remove(node)
 
 
+class HandleCodeBlocks(Transform):
+    """
+    Move doctest blocks out of blockquotes and connect adjacent code blocks.
+    """
+    default_priority = 210
+
+    def apply(self):
+        for node in self.document.traverse(nodes.block_quote):
+            if len(node.children) == 1 and isinstance(node.children[0],
+                                                      nodes.doctest_block):
+                node.replace_self(node.children[0])
+
+
 class MyStandaloneReader(standalone.Reader):
     """
     Add our own transforms.
@@ -136,7 +149,7 @@ class MyStandaloneReader(standalone.Reader):
     def get_transforms(self):
         tf = standalone.Reader.get_transforms(self)
         return tf + [DefaultSubstitutions, MoveModuleTargets,
-                     FilterMessages]
+                     FilterMessages, HandleCodeBlocks]
 
 
 class MyContentsFilter(ContentsFilter):
