@@ -10,6 +10,7 @@
     :license: BSD.
 """
 
+import re
 import sys
 import time
 import StringIO
@@ -21,7 +22,10 @@ from docutils import nodes
 from docutils.parsers.rst import directives
 
 from sphinx.builder import Builder
+from sphinx.util.console import bold
 
+blankline_re = re.compile(r'^\s*<BLANKLINE>', re.MULTILINE)
+doctestopt_re = re.compile(r'#\s*doctest:.+$', re.MULTILINE)
 
 # set up the necessary directives
 
@@ -30,23 +34,33 @@ def test_directive(name, arguments, options, content, lineno,
     # use ordinary docutils nodes for test code: they get special attributes
     # so that our builder recognizes them, and the other builders are happy.
     code = '\n'.join(content)
+    test = None
+    if name == 'doctest':
+        if '<BLANKLINE>' in code:
+            # convert <BLANKLINE>s to ordinary blank lines for presentation
+            test = code
+            code = blankline_re.sub('', code)
+        if doctestopt_re.search(code):
+            if not test:
+                test = code
+            code = doctestopt_re.sub('', code)
     nodetype = nodes.literal_block
-    if name == 'testsetup' or \
-       (name == 'testoutput' and 'hide' in options):
+    if name == 'testsetup' or 'hide' in options:
         nodetype = nodes.comment
-    node = nodetype(code, code)
-    node.line = lineno
     if arguments:
         groups = [x.strip() for x in arguments[0].split(',')]
     else:
         groups = ['default']
-    node['testnodetype'] = name
-    node['groups'] = groups
+    node = nodetype(code, code, testnodetype=name, groups=groups)
+    node.line = lineno
+    if test is not None:
+        # only save if it differs from code
+        node['test'] = test
     if name == 'testoutput':
         # don't try to highlight output
         node['language'] = 'none'
     node['options'] = {}
-    if name == 'testoutput' and 'options' in options:
+    if name in ('doctest', 'testoutput') and 'options' in options:
         # parse doctest-like output comparison flags
         option_strings = options['options'].replace(',', ' ').split()
         for option in option_strings:
@@ -98,8 +112,8 @@ class TestGroup(object):
             raise RuntimeError('invalid TestCode type')
 
     def __repr__(self):
-        return 'TestGroup(name=%r, setup=%r, code=%r)' % (
-            self.name, self.setup, self.code)
+        return 'TestGroup(name=%r, setup=%r, tests=%r)' % (
+            self.name, self.setup, self.tests)
 
 
 class TestCode(object):
@@ -139,6 +153,15 @@ class DocTestBuilder(Builder):
         self.opt = doctest.DONT_ACCEPT_TRUE_FOR_1 | doctest.ELLIPSIS | \
                    doctest.IGNORE_EXCEPTION_DETAIL
 
+        # HACK HACK HACK
+        # doctest compiles its snippets with type 'single'. That is nice
+        # for doctest examples but unusable for multi-statement code such
+        # as setup code -- to be able to use doctest error reporting with
+        # that code nevertheless, we monkey-patch the "compile" it uses.
+        doctest.compile = self.compile
+
+        self.type = 'single'
+
         self.total_failures = 0
         self.total_tries = 0
         self.setup_failures = 0
@@ -162,9 +185,6 @@ Results of doctest builder run on %s
     def get_outdated_docs(self):
         return self.env.all_docs
 
-    def prepare_writing(self, docnames):
-        return
-
     def finish(self):
         # write executive summary
         def s(v):
@@ -182,14 +202,23 @@ Doctest summary
 
         sys.path[0:0] = self.config.doctest_path
 
-    def write_doc(self, docname, doctree):
+    def write(self, build_docnames, updated_docnames, method='update'):
+        if build_docnames is None:
+            build_docnames = self.env.all_docs
+
+        self.info(bold('running tests...'))
+        for docname in build_docnames:
+            # no need to resolve the doctree
+            doctree = self.env.get_doctree(docname)
+            self.test_doc(docname, doctree)
+
+    def test_doc(self, docname, doctree):
         groups = {}
         add_to_all_groups = []
         self.setup_runner = SphinxDocTestRunner(verbose=False,
                                                 optionflags=self.opt)
         self.test_runner = SphinxDocTestRunner(verbose=False,
                                                optionflags=self.opt)
-        self.info()
         if self.config.doctest_test_doctest_blocks:
             def condition(node):
                 return (isinstance(node, (nodes.literal_block, nodes.comment))
@@ -200,10 +229,10 @@ Doctest summary
                 return isinstance(node, (nodes.literal_block, nodes.comment)) \
                         and node.has_key('testnodetype')
         for node in doctree.traverse(condition):
-            code = TestCode(node.astext(),
+            code = TestCode(node.has_key('test') and node['test'] or node.astext(),
                             type=node.get('testnodetype', 'doctest'),
                             lineno=node.line, options=node.get('options'))
-            node_groups = node.get('groups', ['doctest_block'])
+            node_groups = node.get('groups', ['default'])
             if '*' in node_groups:
                 add_to_all_groups.append(code)
                 continue
@@ -217,8 +246,7 @@ Doctest summary
         if not groups:
             return
 
-        self.outfile.write('\nDocument: %s\n----------%s\n' %
-                           (docname, '-'*len(docname)))
+        self._out('\nDocument: %s\n----------%s\n' % (docname, '-'*len(docname)))
         for group in groups.itervalues():
             self.test_group(group, self.env.doc2path(docname, base=None))
         # Separately count results from setup code
@@ -230,6 +258,9 @@ Doctest summary
             self.total_failures += res_f
             self.total_tries += res_t
 
+    def compile(self, code, name, type, flags, dont_inherit):
+        return compile(code, name, self.type, flags, dont_inherit)
+
     def test_group(self, group, filename):
         ns = {}
         examples = []
@@ -242,6 +273,7 @@ Doctest summary
                                             filename, 0, None)
             setup_doctest.globs = ns
             old_f = self.setup_runner.failures
+            self.type = 'exec' # the snippet may contain multiple statements
             self.setup_runner.run(setup_doctest, out=self._out,
                                   clear_globs=False)
             if self.setup_runner.failures > old_f:
@@ -255,14 +287,23 @@ Doctest summary
                     self._out('WARNING: no examples in doctest block at '
                               + filename + ', line %s' % code[0].lineno)
                     continue
+                for example in test.examples:
+                    # apply directive's comparison options
+                    new_opt = code[0].options.copy()
+                    new_opt.update(example.options)
+                    example.options = new_opt
+                self.type = 'single' # ordinary doctests
             else:
                 output = code[1] and code[1].code or ''
-                options = code[1] and code[1].options or None
+                options = code[1] and code[1].options or {}
+                # disable <BLANKLINE> processing as it is not needed
+                options[doctest.DONT_ACCEPT_BLANKLINE] = True
                 example = doctest.Example(code[0].code, output,
                                           lineno=code[0].lineno,
                                           options=options)
                 test = doctest.DocTest([example], {}, group.name,
                                        filename, code[0].lineno, None)
+                self.type = 'exec' # multiple statements again
             # DocTest.__init__ copies the globs namespace, which we don't want
             test.globs = ns
             # also don't clear the globs namespace after running the doctest
@@ -271,11 +312,13 @@ Doctest summary
 
 def setup(app):
     app.add_directive('testsetup', testsetup_directive, 1, (0, 1, 1))
-    app.add_directive('doctest', doctest_directive, 1, (0, 1, 1))
-    app.add_directive('testcode', testcode_directive, 1, (0, 1, 1))
+    app.add_directive('doctest', doctest_directive, 1, (0, 1, 1),
+                      hide=directives.flag, options=directives.unchanged)
+    app.add_directive('testcode', testcode_directive, 1, (0, 1, 1),
+                      hide=directives.flag)
     app.add_directive('testoutput', testoutput_directive, 1, (0, 1, 1),
                       hide=directives.flag, options=directives.unchanged)
     app.add_builder(DocTestBuilder)
     # this config value adds to sys.path
     app.add_config_value('doctest_path', [], False)
-    app.add_config_value('doctest_test_doctest_blocks', True, False)
+    app.add_config_value('doctest_test_doctest_blocks', 'default', False)
